unit tfuncs2;

// Copyright  2002 by Ziff Davis Media, Inc.
// Written by Neil J. Rubenking
{$warn UNIT_PLATFORM off}
{$warn SYMBOL_PLATFORM off}
{$warn UNSAFE_CODE off}
{$warn UNSAFE_TYPE off}
{$warn UNSAFE_CAST off}

interface

uses Windows, SysUtils;

function CompFileTime(const T1, T2: TFileTime): Longint;
function EmptyTime(T : TFileTime) : Boolean;
function GetFileDateTimes(const S : String; VAR vCreTime,
  vModTime : TFileTime; VAR vAttr : DWORD) : Boolean; overload;
function GetFileDateTimes(const S : String; VAR vCreTime,
  vModTime : TFileTime) : Boolean; overload;
function GetRemote(const S : String) : String;

const
  c_second  = 1000/MSecsPerDay;
  ss : ARRAY[Boolean] OF String = ('', 's');
VAR
  timeFmt  : String;
  dateFmt  : String;
  wholeFmt : String;

implementation

uses shlobj, ShellApi, ActiveX;

function CompFileTime(const T1, T2: TFileTime): Longint;
// A TFileTime is a 64-bit quantity. FileTouch uses a TFileTime
// with both halves set to $FFFFFFFF to represent a non-existent
// filetime. If both are non-existent, return 0 for equal. If just
// one is non-existent, return 1 or -1 as appropriate. Otherwise,
// if they're within half a second return 0; if not call
// CompareFileTime.
begin
  IF EmptyTime(T1) THEN
    IF EmptyTime(T2) THEN
      Result := 0
    ELSE Result := -1
  ELSE
    IF EmptyTime(T2) THEN
      Result := 1
    ELSE
      begin
        // The unit of TFileTime is 100 nanoseconds. It takes 5
        // million of those to make a half-second
        IF (T1.dwHighDateTime = T2.dwHighDateTime) AND
          (Abs(T1.dwLowDateTime-T2.dwLowDateTime) < 5000000) THEN
          Result := 0
        ELSE Result := CompareFileTime(T1, T2);
      end;
end;

function EmptyTime(T : TFileTime) : Boolean;
begin
  Result := (T.dwLowDateTime = DWORD(-1)) AND
            (T.dwHighDateTime = DWORD(-1));
end;

function GetFileDateTimes(const S : String; VAR vCreTime,
  vModTime : TFileTime; VAR vAttr : DWORD) : Boolean; Overload;
// Get both date/timestamps and attributes
VAR
  FindData : TWin32FindData;
  FindHan  : THandle;
begin
  Result := False;
  FillChar(FindData, SizeOf(FindData), 0);
  FindHan := FindFirstFile(PChar(S), FindData);
  IF FindHan = INVALID_HANDLE_VALUE THEN Exit
  ELSE
    try
      WITH FindData DO
        begin
          vCreTime := ftCreationTime;
          vModTime := ftLastWriteTime;
          vAttr    := dwFileAttributes;
        end;
    finally
      Windows.FindClose(FindHan);
    end;
  Result := True;
end;

function GetFileDateTimes(const S : String; VAR vCreTime,
  vModTime : TFileTime) : Boolean; overload;
// Get the date/time stamps without worrying about attributes
VAR Dummy : DWord;
begin
  Result := GetFileDateTimes(S, vCreTime,vModTime, Dummy);
end;

function GetRemote(const S : String) : String;
VAR
  NameBuff : ARRAY[0..MAX_PATH] OF Char;
  Siz      : DWORD;
begin
  Siz := MAX_PATH;
  IF WNetGetConnection(PChar(ExtractFileDrive(S)), @NameBuff,
    Siz) = NO_ERROR THEN
      Result := StrPas(NameBuff)
  ELSE Result := S;
end;

procedure InitDateTimeFormats;
// This routine takes the locally defined date and time formats,
// retaining the order of their items, and forces them to use
// four-digit years and two-digit days, months, and hours. This
// makes for better alignment in the display

  function Exactly(hiC : Char; num : Integer; hiS : String) : String;
  // Find the specified character within the string and replace
  // a continuous string of that character with exactly the number
  // specified. E.g. Exactly('y', 4, 'm/d/y') would return 'm/d/yyyy'
  VAR
    P : Integer;
    N : Integer;
    C : Char;
    S : String;
  begin
    C := lowercase(hiC)[1];
    S := lowercase(hiS);
    P := Pos(C, S);
    IF P = 0 THEN Result := S
    ELSE
      begin
        N := P;
        REPEAT
          Inc(N);
        UNTIL (N > Length(S)) OR (S[N]<>C);
        Result := StringReplace(S, StringOfChar(C, N-P),
          StringOfChar(C, num), [rfIgnoreCase, rfReplaceAll]);
      end;
  end;
begin
  DateFmt := Exactly('y', 4,
              Exactly('d', 2,
                Exactly('m', 2, ShortDateFormat)));
  TimeFmt := Exactly('h', 2, LongTimeFormat);
  WholeFmt := DateFmt + ' ' + TimeFmt;
  // Now modify them for use with DateTimePicker
  DateFmt := StringReplace(DateFmt, 'm', 'M',
    [rfIgnoreCase, rfReplaceAll]);
  TimeFmt := StringReplace(TimeFmt, 'ampm', 'tt',
    [rfIgnoreCase, rfReplaceAll]);
  TimeFmt := StringReplace(TimeFmt, 'am/pm', 'tt',
    [rfIgnoreCase, rfReplaceAll]);
  TimeFmt := StringReplace(TimeFmt, 'a/p', 'tt',
    [rfIgnoreCase, rfReplaceAll]);
  IF Pos('tt', TimeFmt) = 0 THEN
    TimeFmt := StringReplace(TimeFmt, 'h', 'H',
      [rfIgnoreCase, rfReplaceAll]);
end;

initialization
  InitDateTimeFormats;
end.
