program ftouch;

// Copyright  2002 by Ziff-Davis, Inc.
// Written by Neil J. Rubenking

// We need the *.RES entry in order to easily add Version info, but
// we need a separate .res file to give this console app a specific
// icon. The result is "[Error] WANING. Duplicate resource(s)".
// Simply ignore this error.
{$R 'fres.res' 'fres.rc'}
{$R *.RES}
{$APPTYPE CONSOLE}
{$warn UNIT_PLATFORM off}
{$warn SYMBOL_PLATFORM off}
{$warn UNSAFE_CODE off}
{$warn UNSAFE_TYPE off}
{$warn UNSAFE_CAST off}




uses
  Windows,
  SysUtils,
  StrUtils,
  Classes,
  inifiles,
  sharedu,
  ShellApi,
  Registry,
  ftObju,
  tfuncs2;

VAR
  T    : TShared;
  Main : String;
  Path : String;

procedure DOAbout(T : TShared);
VAR
  Files : TStringList;
  TS    : TStringList;
  N     : Integer;
  AttrAdd : DWORD;
  AttrRem : DWORD;
  Rslt : Integer;

  procedure AddFiles(const S : String);
  VAR
    FindHan  : THandle;
    FindData : TWin32FindData;
    Path     : String;
  begin
    Path := ExpandFileName(S);
    FindHan := FindFirstFile(PChar(Path), FindData);
    Path := IncludeTrailingBackslash(ExtractFileDir(Path));
    IF FindHan = INVALID_HANDLE_VALUE THEN Exit
    ELSE
      try
        REPEAT
          IF StrComp(FindData.cFilename, '.') = 0 THEN Continue;
          IF StrComp(FindData.cFilename, '..') = 0 THEN Continue;
          IF Files.IndexOf(Path+FindData.cFilename) >= 0 THEN Continue;
          Files.AddObject(Path+FindData.cFileName, TftObj.Create(
            FindData.dwFileAttributes, Path+FindData.cFileName,
            FindData.ftCreationTime, FindData.ftLastWriteTime));
        UNTIL NOT FindNextFile(FindHan, FindData);
      finally
        Windows.FindClose(FindHan);
      end;
  end;

  procedure LoadFromPreset(which : Integer);
  VAR Sec : String;
    function AtVal(I :Integer) : Integer;
    begin
      CASE I OF
        1 :  Result := -1;
        2 :  Result := 1;
        ELSE Result := 0;
      end;
    end;

    function SeqFrom(Num, Uni : Integer) : Integer;
    begin
      CASE Num OF
        0 :  Result := 1;
        1 :  Result := 2;
        2 :  Result := 5;
        3 :  Result := 10;
        4 :  Result := 20;
        5 :  Result := 30;
        ELSE Result := 0;
      END;
      CASE Uni OF
        0 : ;
        1 :  Result := Result * 60;
        ELSE Result := 0;
      END;
    end;
  begin
    Sec := Format('Preset%d', [which]);
    WITH TIniFile.Create(ChangeFileExt(Main, '.INI')) DO
      begin // Read the preset into the other fields of T
        T.AtA := AtVal(ReadInteger(sec, 'AttrA', 0));
        T.AtS := AtVal(ReadInteger(sec, 'AttrS', 0));
        T.AtH := AtVal(ReadInteger(sec, 'AttrH', 0));
        T.AtR := AtVal(ReadInteger(sec, 'AttrR', 0));
        T.ModAct := ReadInteger(sec, 'Mod', 0);
        T.CreAct := ReadInteger(sec, 'Cre', 0);
        T.SetDate := Now;
        T.SetTime := Now;
        CASE ReadInteger(sec, 'Clock', 0) OF
          0 : ;
          1 : T.ZerSec := True;
          2 : T.ZerMin := True;
          ELSE begin
            T.SetDate := ReadDate(sec, 'Date', Now);
            T.SetTime := ReadTime(sec, 'Time', Now);
          end;
        END;
        T.ExDate := NOT ReadBool(sec, 'UseDate', True);
        T.ExTime := NOT ReadBool(sec, 'UseTime', True);
        T.Seq := SeqFrom(ReadInteger(sec, 'SeqNum', 0),
                         ReadInteger(sec, 'SeqUnit', 0));
      end;
  end;

  function ZerTime(T : TDateTime; MinToo : Boolean)
    : TDateTime;
  VAR Ye, Mo, Da, Ho, Mi, Se, Ms : Word;
  begin
    DecodeDate(T, Ye, Mo, Da);
    DecodeTime(T, Ho, Mi, Se, Ms);
    IF MinToo THEN
      Result := EncodeDate(Ye, Mo, Da) + EncodeTime(Ho, 0, 0, 0)
    ELSE
      Result := EncodeDate(Ye, Mo, Da) + EncodeTime(Ho, Mi, 0, 0);
  end;

  procedure SetMask(Val : Integer; Bit : DWord);
  begin
    CASE Val OF
      -1 : AttrRem := AttrRem AND (NOT bit);
      1  : AttrAdd := AttrAdd OR bit;
    END;
  end;

  procedure Advance(VAR vT, vD: TDateTime);
  // Advance the time by the specified sequential value. If the
  // time passed 24 hours (i.e. 1.0), bring it back into the
  // 0.0-1.0 range and add one to the date.
  VAR TWas : Extended;
  begin
    TWas := Int(vT);
    vT := vT+T.Seq*c_second;
    IF Int(vT) > TWas THEN
      begin
        vT := vT-1;
        IF vD <> 0 THEN
          vD := vD+1;
      end;
  end;

begin
  Files := TStringList.Create;
  Files.Sorted := True;
  try
    TS := TStringList.Create;
    try
      TS.Text := T.Specs;
      FOR N := 0 TO TS.Count-1 DO
        AddFiles(TS[N]);
    finally
      TS.Free;
    end;
  IF T.Preset > 0 THEN
    LoadFromPreset(T.Preset);
  IF T.SetTime = -1 THEN
    begin
      IF T.ZerSec THEN
        T.SetTime := ZerTime(Now, False)
      ELSE IF T.ZerMin THEN
        T.SetTime := ZerTime(Now, True)
      ELSE T.SetTime := Now;
    end;
  IF T.SetDate = -1 THEN T.SetDate := Now;
  IF T.ExDate THEN T.SetDate := -1;
  IF T.ExTime THEN T.SetTime := -1;
  // Build the add/remove attribute masks
  AttrRem := DWORD(-1);
  AttrAdd := DWORD(0);
  SetMask(T.AtA, FILE_ATTRIBUTE_ARCHIVE);
  SetMask(T.AtS, FILE_ATTRIBUTE_SYSTEM);
  SetMask(T.AtH, FILE_ATTRIBUTE_HIDDEN);
  SetMask(T.AtR, FILE_ATTRIBUTE_READONLY);
  FOR N := 0 TO Files.Count-1 DO
    begin
      WITH tftObj(Files.Objects[N]) DO
        begin
          CASE T.ModAct OF
            c_fromsys   : SetModPendingOpt(T.SetDate, T.SetTime);
            c_fromother : ModPending := CreTime;
          END;
          CASE T.CreAct OF
            c_fromsys   : SetCrePendingOpt(T.SetDate, T.SetTime);
            c_fromother : CrePending := ModTime;
          END;
          IF ((T.ModAct = c_fromSys) OR (T.CreAct = c_fromSys)) AND
            (T.Seq >0) THEN
            Advance(T.SetTime, T.SetDate);
          IF (AttrRem <> DWORD(-1)) OR (AttrAdd <> DWORD(0)) THEN
            AttrChange(AttrRem, AttrAdd);
          // **** HERE **** is where the action is. PerformPending
          // applies the other changes that have been set up by the
          // lines preceding
          Rslt := PerformPending;
          // ********************
          CASE T.Rept OF
            0 : ; // no report
            1 : IF Rslt > 0 THEN
                  WriteLn('Error: ', Files[N]);
            2 : IF (Rslt = -1) OR (Rslt = 0) THEN
                  WriteLn(Files[N])
                ELSE WriteLn('Error: ', Files[N]);
            3 : begin
              IF (Rslt = -1) OR (Rslt = 0) THEN
                begin
                  WriteLn(Files[N]);
                  WriteLn(Report);
                end
              ELSE
                begin
                  WriteLn(Files[N]);
                  WriteLn(Status);
                end;
              WriteLn;
            end;
          END;
        end;
    end;
  finally
    Files.Free;
  end;
end;

begin
  SetLength(Path, MAX_PATH);
  SetLength(Path, GetCurrentDirectory(MAX_PATH, PChar(Path)));
  Main := '';
  WITH TRegistry.Create DO
  try
    RootKey := HKEY_LOCAL_MACHINE;
    IF OpenKeyReadOnly('SOFTWARE\Microsoft\Windows\CurrentVersion\App Paths\FileTouch.EXE') THEN
      IF ValueExists('') THEN
        Main := ReadString('');
  finally
    Free;
  end;
  IF Main = '' THEN
    begin
      WriteLn('ERROR: Cannot find FileTouch.exe in Registry. Please reinstall FileTouch.');
      Halt(3);
    end;
  IF NOT FileExists(Main) THEN
    begin
      WriteLn('ERROR: FileTouch.exe is not present at the registered location:');
      WriteLn(Main);
      WriteLn('Please reinstall FileTouch.');
      Halt(2);
    end;
  ZeroSwitches(T);
  IF ParamCount > 0 THEN
    begin
      IF (ParamStr(1)='/?') OR (ParamStr(1)='-?') THEN
        begin
          ShowSyntax(True);
          Halt;
        end
      ELSE IF Analyze(T, ChangeFileExt(Main, '.INI'), True) THEN
        begin
          IF (T.Specs <> '') AND T.AnySwitch AND (NOT T.PresetF) THEN
            begin
              DOAbout(T);
              Halt;
            end;
        end
      ELSE Halt(1);
    end
  ELSE
    begin
      ShowSyntax(True);
      Halt;
    end;
  ShellExecute(0, nil, PChar(Main), PChar(Perfect(T)),
      PChar(Path), SW_SHOW);
end.
