unit presetu;

// Copyright  2002 by Ziff Davis Media, Inc.
// Written by Neil J. Rubenking
{$warn UNIT_PLATFORM off}
{$warn SYMBOL_PLATFORM off}
{$warn UNSAFE_CODE off}
{$warn UNSAFE_TYPE off}
{$warn UNSAFE_CAST off}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics,
  Controls, Forms, Dialogs, ImgList, ActnList, ComCtrls, ToolWin,
  StdCtrls, AppEvnts;

type
  TPresetForm = class(TForm)
    alPresets               : TActionList;
      acLblM                : TAction;
      acLblC                : TAction;
      acLblA                : TAction;
      acLblS                : TAction;
      acLblH                : TAction;
      acLblR                : TAction;
      acSeq                 : TAction;
      acDate                : TAction;
      acTime                : TAction;
      acHelp                : TAction;
    ebName                  : TEdit;
    ebDesc                  : TEdit;
    gbAttr                  : TGroupBox;
      lblA                  : TLabel;
      lblS                  : TLabel;
      lblH                  : TLabel;
      lblR                  : TLabel;
      tbAttrA               : TToolBar;
        tbAminus            : TToolButton;
        tbAPlus             : TToolButton;
      tbAttrS               : TToolBar;
        tbSMinus            : TToolButton;
        tbSPlus             : TToolButton;
      tbAttrH               : TToolBar;
        tbHMinus            : TToolButton;
        tbHPlus             : TToolButton;
      tbAttrR               : TToolBar;
        tbRMinus            : TToolButton;
        tbRPlus             : TToolButton;
    gbStamp                 : TGroupBox;
      lblM                  : TLabel;
      lblC                  : TLabel;
      tbMod                 : TToolBar;
        tbModFmClk          : TToolButton;
        tbModFmCre          : TToolButton;
        tbModFmFld          : TToolButton;
      tbCre                 : TToolBar;
        tbCreFmClk          : TToolButton;
        tbCreFmMod          : TToolButton;
        tbCreFmFld          : TToolButton;
    gbClock                 : TGroupBox;
      cbDatex               : TCheckBox;
      DTDateMod             : TDateTimePicker;
      cbTimex               : TCheckBox;
      DTTimeMod             : TDateTimePicker;
      cbSeq                 : TCheckBox;
      comSeqN               : TComboBox;
      comSeqU               : TComboBox;
      tbTimes               : TToolBar;
        tbtSpec             : TToolButton;
        tbtSys              : TToolButton;
        tbtZMin             : TToolButton;
        tbtZSec             : TToolButton;
    btnHelp                 : TButton;
    btnCancel               : TButton;
    btnOK                   : TButton;
    sbPre                   : TStatusBar;
    ilTime                  : TImageList;
    ilTimeDim               : TImageList;
    aePreset                : TApplicationEvents;
    procedure FormCreate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure acLabelsExecute(Sender: TObject);
    procedure acTimeExecute(Sender: TObject);
    procedure acDateExecute(Sender: TObject);
    procedure acSeqExecute(Sender: TObject);
    procedure acHelpExecute(Sender: TObject);
    procedure LabelsClick(Sender: TObject);
    procedure FlyoverMouseMove(Sender: TObject; Shift: TShiftState;
      X, Y: Integer);
    procedure TimeBtnsClick(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure aePresetSettingChange(Sender: TObject;
      Flag: Integer; const Section: String; var Result: Integer);
  private
    { Private declarations }
    fWhich    : Integer;
    ininame   : String;
    sec       : String;
    FirstTime : Boolean;
    procedure EnDisDateTime;
  public
    { Public declarations }
    procedure SetData(which : Integer);
    procedure GetData(VAR vName, vDesc : String);
  end;

var
  PresetForm: TPresetForm;

implementation
uses inifiles, formposu, tfuncsu, tfuncs2, ftobju, mainu, dateutils;

{$R *.dfm}

procedure TPresetForm.FormCreate(Sender: TObject);
begin
  FirstTime        := True;
  IniName          := ChangeFileExt(Application.ExeName, '.INI');
  Self.Font        := MainForm.Font;
  DTDateMod.Format := DateFmt;
  DTTimeMod.Format := TimeFmt;
  cbSeq.Caption    := '';
  cbDateX.Caption  := '';
  cbTimeX.Caption  := '';
  GetPosFmINI(IniName, Self, True);
  LoadImageList(ilTime, [24, 15, 16, 17]);
  LoadImageList(ilTimeDim, 55, 58);
end;

procedure TPresetForm.FormActivate(Sender: TObject);
VAR dummy : Integer;
begin
  IF FirstTime THEN
    begin
      Show;
      // This activates the code that gets the messagebox font
      // for the richedit and the icon title font for the listview
      aePresetSettingChange(aePreset, 0, '', Dummy);
    end;
end;

procedure TPresetForm.acLabelsExecute(Sender: TObject);
// Clicking on one of the labels in the Attributes or Settings
// boxes cycles through the available states of the adjacent
// toolbar (each button down or NO buttons down)
begin
  CASE (Sender AS TAction).Tag OF
    1 : LabelsClick(lblM);
    2 : LabelsClick(lblC);
    3 : LabelsClick(lblA);
    4 : LabelsClick(lblS);
    5 : LabelsClick(lblH);
    6 : LabelsClick(lblR);
  END;
end;

procedure TPresetForm.acTimeExecute(Sender: TObject);
// When the time checkbox is clicked, toggle its state and enable-
// disable other controls as appropriate. If the box is checked
// and the time-control is enabled, give focus to the time control
begin
  WITH Sender AS TAction DO
    begin
      Checked := NOT Checked;
      EnDisDateTime;
      IF Checked AND DTTimeMod.Enabled THEN
        DTTimeMod.SetFocus;
    end;
end;

procedure TPresetForm.acDateExecute(Sender: TObject);
// When the date checkbox is clicked, toggle its state and enable-
// disable other controls as appropriate. If the box is checked
// and the date-control is enabled, give focus to the date-control
begin
  WITH Sender AS TAction DO
    begin
      Checked := NOT Checked;
      EnDisDateTime;
      IF Checked AND DTDateMod.Enabled THEN
        DTDateMod.SetFocus;
    end;
end;

procedure TPresetForm.acSeqExecute(Sender: TObject);
// When the sequential checkbox is clicked, toggle its state and
// enable-disable other controls as appropriate.
begin
  WITH Sender AS TAction DO
    begin
      Checked := NOT Checked;
      EnDisDateTime;
    end;
end;

procedure TPresetForm.acHelpExecute(Sender: TObject);
begin
  Application.HelpContext(14);
end;

procedure TPresetForm.LabelsClick(Sender: TObject);
begin
  WITH Sender AS TLabel DO
   CycleButtons(FocusControl AS TToolBar);
end;

procedure TPresetForm.FlyoverMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
// This method exists to enable flyover help for disabled controls
VAR
  P : TPoint;
  N : Integer;
begin
  P := Point(X, Y);
  WITH Sender AS TWinControl DO
    begin
      Hint := '';
      FOR N := 0 TO ControlCount-1 DO
        IF Controls[N] IS TWinControl THEN
          IF PtInRect(TWinControl(Controls[N]).BoundsRect, P) THEN
            Hint := TWinControl(Controls[N]).Hint;
      IF Hint = '' THEN
        begin
          // If the mouse is over one of the groupboxes but not over
          // any control, show a hint about the buttons
          IF (Sender AS TWinControl).Tag = 1 THEN
            Hint := '|Click a button to select; click again to '+
              'de-select'
          ELSE
            Application.CancelHint;
        end;
    end;
end;

procedure TPresetForm.TimeBtnsClick(Sender: TObject);
// The date/time controls are enabled only if the "Specified
// date/time" button is down
begin
  EnDisDateTime;
end;

procedure TPresetForm.btnOKClick(Sender: TObject);
// Check to make sure all the settings are reasonable. If not explain
// what's wrong. If so, return with modal result "OK"
VAR
  N     : Integer;
  fName : String;

  function NumFor(B1, B2, B3 : Boolean) : Integer;
  begin
    IF B1      THEN Result := 1
    ELSE IF B2 THEN Result := 2
    ELSE IF B3 THEN Result := 3
    ELSE            Result := 0;
  end;
begin
  IF NOT (tbModFmClk.Down OR tbModFmCre.Down OR tbModFmFld.Down OR
    tbCreFmClk.Down OR tbCreFmMod.Down OR tbCreFmFld.Down OR
    tbAMinus.Down OR tbAPlus.Down OR tbSMinus.Down OR tbSPlus.Down
    OR tbRMinus.Down OR tbRPlus.Down OR tbHMinus.Down OR
    tbHPlus.Down) THEN
    begin
      MessageBox(Handle, 'You must select at least one action from '+
        'the Attributes or Settings pane.', PChar(Application.Title),
        MB_OK OR MB_ICONSTOP);
      Exit;
    end;
  IF NOT (cbDateX.Checked OR cbTimeX.Checked) THEN
    IF tbModFmClk.Down OR tbCreFmClk.Down THEN
      begin
        MessageBox(Handle, 'In order to set the Modified or '+
          'Created date/time stamp to a specific value, either the '+
          'date, the time, or both must be checked.',
          PChar(Application.Title), MB_OK OR MB_ICONSTOP);
        Exit;
      end;
  IF (ebName.Text = '') OR (ebDesc.Text = '') THEN
    begin
      MessageBox(Handle, 'You must supply a name and description '+
        'for this preset.', PChar(Application.Title),
        MB_OK OR MB_ICONSTOP);
      Exit;
    end;
  WITH TIniFile.Create(ininame) DO
  try
    // Check the names of existing presets
    FOR N := 1 TO 5 DO
      begin
        // Skip the one we're working on
        IF N = fWhich THEN Continue;
        fName := ReadString(Format('Preset%d', [N]), 'Name', '');
        IF AnsiCompareText(ebName.Text, fName) = 0 THEN
          begin
            MessageBox(Handle, PChar(Format('Preset #%d is named '+
              '"%s". Please choose a different name.', [N, fName])),
              PChar(Application.Title), MB_OK OR MB_ICONSTOP);
            Exit;
          end;
      end;
    // WHEW! Everything is OK, so write the data to the INI file
    EraseSection(Sec);
    WriteBool   (Sec, 'Enabled', True);
    WriteString (sec, 'Name',    ebName.Text);
    WriteString (sec, 'Desc',    ebDesc.Text);
    WriteDate   (sec, 'Date',    DTDateMod.Date);
    WriteTime   (sec, 'Time',    DTTimeMod.Time);
    WriteBool   (sec, 'UseDate', cbDateX.Checked);
    WriteBool   (sec, 'UseTime', cbTimeX.Checked);
    WriteBool   (sec, 'UseSeq',  cbSeq.Checked);
    WriteInteger(sec, 'SeqNum',  comSeqN.ItemIndex);
    WriteInteger(sec, 'SeqUnit', comSeqU.ItemIndex);
    //2.1 Re-ordering of buttons accomplished by "Pred" below
    WriteInteger(Sec, 'Clock',   Pred(NumFor(tbtSys.Down, //2.1
      tbtZMin.Down, tbtZSec.Down)));                      //2.1
//2.1    WriteInteger(Sec, 'Clock',   NumFor(tbtSys.Down,
//2.1      tbtZMin.Down, tbtZSec.Down));
    WriteInteger(Sec, 'Mod',     NumFor(tbModFmClk.Down,
      tbModFmCre.Down, tbModFmFld.Down));
    WriteInteger(Sec, 'Cre',     NumFor(tbCreFmClk.Down,
      tbCreFmMod.Down, tbCreFmFld.Down));
    WriteInteger(Sec, 'AttrA',   NumFor(tbAMinus.Down,
      tbAPlus.Down, False));
    WriteInteger(Sec, 'AttrS',   NumFor(tbSMinus.Down,
      tbSPlus.Down, False));
    WriteInteger(Sec, 'AttrH',   NumFor(tbHMinus.Down,
      tbHPlus.Down, False));
    WriteInteger(Sec, 'AttrR',   NumFor(tbRMinus.Down,
      tbRPlus.Down, False));
  finally
    Free;
  end;
  ModalResult := mrOK;
end;

procedure TPresetForm.aePresetSettingChange(Sender: TObject;
  Flag: Integer; const Section: String; var Result: Integer);
// Set the form's font to the system's messagebox font
VAR
  NCM : NONCLIENTMETRICS;
  F   : TFont;

  procedure FixTBS;
  // Delphi automatically sizes and positions most controls based
  // on Large/Small fonts, but it doesn't do so for free-floating
  // toolbars.
    procedure Fixup(TB : TToolBar; TL : TLabel);
    begin
      TB.Left := TB.Parent.Width - TB.Width - 8;
      TB.Top  := TL.Top + TL.Height - TB.Height + 2;
      TL.Left := TB.Left - TL.Width - 8;
    end;

    procedure CenterC(TC : TControl);
    begin
      TC.Left := (TC.Parent.ClientWidth - TC.Width) DIV 2;
    end;

  begin
    Fixup(tbAttrA, lblA);
    Fixup(tbAttrS, lblS);
    Fixup(tbAttrH, lblH);
    Fixup(tbAttrR, lblR);
    CenterC(tbMod);
    CenterC(tbCre);
    CenterC(lblM);
    CenterC(lblC);
    tbTimes.Left := DTTimeMod.Left;

    tbTimes.Top  := (DTTimeMod.Top + DTTimeMod.Height + ComSeqN.Top -
      tbTimes.Height) DIV 2;
    lblM.Top  := lblA.Top;
    tbMod.Top := tbAttrS.Top;                                    
    lblC.Top  := lblH.Top;
    tbCre.Top := tbAttrR.Top;
  end;

begin
  F := TFont.Create;
  try
    FillChar(NCM, SizeOf(NCM), 0);
    NCM.cbSize := SizeOf(NCM);
    SystemParametersInfo(SPI_GETNONCLIENTMETRICS, SizeOf(NCM), @NCM,
      0);
    F.Handle := CreateFontIndirect(NCM.lfMessageFont);
    IF (F.Name <> Self.Font.Name) OR (F.Size<>Self.Font.Size) THEN
      begin
        F.Style := [];
        Self.Font.Assign(F);
        Self.Canvas.Font := Self.Font;
        sbPre.Height     := Self.Canvas.TextHeight('Ay') + 6;
        fixTBs;
      end
    ELSE IF FirstTime THEN
      fixTBs;
    FirstTime := False;
  finally
    F.Free;
  end;
end;

procedure TPresetForm.EnDisDateTime;
// Enable/disable controls based on the status of the three check
// boxes and on which of the time-buttons is depressed
begin
  dtDateMod.Enabled := acDate.Checked AND tbtSpec.Down;
  dtTimeMod.Enabled := acTime.Checked AND tbtSpec.Down;
  acSeq.Enabled     := acDate.Checked AND acTime.Checked;
  comSeqN.Enabled   := acSeq.Enabled AND acSeq.Checked;
  comSeqU.Enabled   := acSeq.Enabled AND acSeq.Checked;
  IF acDate.Checked OR acTime.Checked THEN
    begin
      tbModFmClk.Enabled := True;
      tbCreFmClk.Enabled := True;
    end
  ELSE
    begin
      tbModFmClk.Down    := False;
      tbModFmClk.Enabled := False;
      tbCreFmClk.Down    := False;
      tbCreFmClk.Enabled := False
    end;
end;

procedure TPresetForm.SetData(which: Integer);
VAR N : Integer;

  procedure ButtonOne(num : Integer; Btns : ARRAY OF TToolButton);
  begin
    IF num < Low(Btns)  THEN Exit;
    IF num > High(Btns) THEN Exit;
    IF Btns[Num] = nil  THEN Exit;
    WITH Btns[Num] DO
      Down := True;
  end;

  procedure SetFrom(tbTo, tbFm : TToolButton);
  begin
    IF NOT tbFm.Down THEN Exit;
    tbTo.Down := True;
  end;
begin
  Self.Caption := Format('Configure preset #%d', [which]);
  Sec          := Format('Preset%d', [which]);
  fWhich       := Which;
  // Start with all buttons UP
  FOR N := 0 TO ComponentCount-1 DO
    IF Components[N] IS TToolButton THEN
      WITH TToolButton(Components[N]) DO
        Down := False;
  WITH TIniFile.Create(ininame) DO
  try
    IF ReadBool(sec, 'Enabled', False) THEN
      begin
        ebName.Text       := ReadString(sec, 'Name', '');
        ebDesc.Text       := ReadString(sec, 'Desc', '');
        DTDateMod.Date    := ReadDate(sec, 'Date', Now);
        DTTimeMod.Time    := ReadTime(sec, 'Time', Now);
        cbDateX.Checked   := ReadBool(sec, 'UseDate', True);
        cbTimeX.Checked   := ReadBool(sec, 'UseTime', True);
        cbSeq.Checked     := ReadBool(sec, 'UseSeq', False);
        comSeqN.ItemIndex := ReadInteger(sec, 'SeqNum', 0);
        comSeqU.ItemIndex := ReadInteger(sec, 'SeqUnit', 0);
        //2.1 Re-ordering of buttons accomplished by "Succ" below
        ButtonOne(Succ(ReadInteger(sec, 'Clock', 0)), [tbtSpec, tbtSys, //2.1
          tbtZMin, tbtZSec]);                                     //2.1
//2.1        ButtonOne(ReadInteger(sec, 'Clock', 0), [tbtSpec, tbtSys,
//2.1          tbtZMin, tbtZSec]);
        ButtonOne(ReadInteger(sec, 'Mod', 0),
          [nil, tbModFmClk, tbModFmCre, tbModFmFld]);
        ButtonOne(ReadInteger(sec, 'Cre', 0),
          [nil, tbCreFmClk, tbCreFmMod, tbCreFmFld]);
        ButtonOne(ReadInteger(sec, 'AttrA', 0),
          [nil, tbAMinus, tbAPlus]);
        ButtonOne(ReadInteger(sec, 'AttrS', 0),
          [nil, tbSMinus, tbSPlus]);
        ButtonOne(ReadInteger(sec, 'AttrH', 0),
          [nil, tbHMinus, tbHPlus]);
        ButtonOne(ReadInteger(sec, 'AttrR', 0),
          [nil, tbRMinus, tbRPlus]);
      end
    ELSE
      begin
        ebName.Text       := '';
        ebDesc.Text       := '';
        tbtSpec.Down      := True;
        DTDateMod.Date    := MainForm.DTDateMod.Date;
        DTTimeMod.Time    := MainForm.DTTimeMod.Time;
        cbDateX.Checked   := MainForm.cbDateX.Checked;
        cbTimeX.Checked   := MainForm.cbTimeX.Checked;
        cbSeq.Checked     := MainForm.cbSeq.Checked;
        comSeqN.ItemIndex := MainForm.comSeqN.ItemIndex;
        comSeqU.ItemIndex := MainForm.comSeqU.ItemIndex;
        SetFrom(tbModFmClk, MainForm.tbModFmClk);
        SetFrom(tbModFmCre, MainForm.tbModFmCre);
        SetFrom(tbModFmFld, MainFOrm.tbModFmFld);
        SetFrom(tbCreFmClk, MainForm.tbCreFmClk);
        SetFrom(tbCreFmMod, MainForm.tbCreFmMod);
        SetFrom(tbCreFmFld, MainForm.tbCreFmFld);
        SetFrom(tbAMinus  , MainForm.tbAMinus);
        SetFrom(tbAPlus   , MainForm.tbAPlus);
        SetFrom(tbSMinus  , MainForm.tbSMinus);
        SetFrom(tbSPlus   , MainForm.tbSPlus);
        SetFrom(tbHMinus  , MainForm.tbHMinus);
        SetFrom(tbHPlus   , MainForm.tbHPlus);
        SetFrom(tbRMinus  , MainForm.tbRMinus);
        SetFrom(tbRPlus   , MainForm.tbRPlus);
      end;
  finally
    Free;
  end;
  EnDisDateTime;
end;

procedure TPresetForm.GetData(VAR vName, vDesc : String);
begin
  vName := ebName.Text;
  vDesc := ebDesc.Text;
end;

end.
