unit sharedu;

// Copyright  1999, 2002 by Ziff Davis Media, Inc.
// Written by Neil J. Rubenking
{$warn UNIT_PLATFORM off}
{$warn SYMBOL_PLATFORM off}
{$warn UNSAFE_CODE off}
{$warn UNSAFE_TYPE off}
{$warn UNSAFE_CAST off}

interface

uses SysUtils, Classes, inifiles, StrUtils, Windows, Messages;

TYPE
  PShared = ^TShared;
  TShared = Record
    AppHandle : THandle;
    WinHandle : THandle;
    Rept               : Integer;
    Preset             : Integer;
    PresetF            : Boolean;
    AtA, AtS, AtH, AtR : Integer;
    ModAct, CreAct     : Integer;
    SetTime, SetDate   : TDateTime;
    ZerSec, ZerMin     : Boolean;
    ExDate, ExTime     : Boolean;
    Seq                : Integer;
    AnySwitch          : Boolean;
    Specs              : ShortString;
  end;
VAR
  hMap : THandle;
  PH   : ^TShared;
CONST
  MainMapName : PChar = 'PC Magazine FileTouch'+
    '{570CA000-0B2F-4C3C-98D7-C0EFF1CFEFF4}';
  WM_HEYYOU = WM_USER + 2;
  c_fromsys = 1;
  c_fromother = 2;

function Analyze(VAR T : TShared; const ininame : String;
  CanWrite : Boolean) : Boolean;
procedure CopyShared(const fromT : TShared; VAR toT : TShared);
function Perfect(T : TShared) : String;
procedure ShowSyntax(CanWrite : Boolean);
procedure ZeroAll(VAR T : TShared);
procedure ZeroSwitches(VAR T : TShared);

implementation
uses tFuncs2;

function Analyze(VAR T : TShared; const ininame : String;
  CanWrite : Boolean) : Boolean;
// Analyze the command-line and parse its switches into the shared
// data area T. Return True if the command line is OK. If there are
// errors, output depends on whether FileTouch or FTOUCH is handling
// the command-line. If FTOUCH, just write to the console. If
// FileTouch, accumulate them and present in a messagebox.
VAR
  N : Integer;
  Msg : String;

  procedure WriteLx(const S : String);
  begin
    IF CanWrite THEN System.WriteLn(S)
    ELSE Msg := Msg + S + #13#10;
  end;

  function CheckPreset(which : Integer) : Boolean;
  begin
    WITH TIniFile.Create(ininame) DO
    try
      IF ReadBool(Format('Preset%d', [which]), 'Enabled', False) THEN
        begin
          Result := True;
          IF (ReadInteger(Format('Preset%d',[which]), 'Cre', 0)=3) OR
             (ReadInteger(Format('Preset%d',[which]), 'Mod', 0)=3) THEN
            T.PresetF := True;
        end
      ELSE
        begin
          WriteLx(Format('Preset # %d is not configured.', [which]));
          Result := False;
        end;
    finally
      Free;
    end;
  end;

  function GoodSwitch(const Switch : String) : Boolean;
  VAR
    S : String;
    P : Integer;
  const twos =
    '^A+^S+^H+^R+^A-^S-^H-^R-^MS^CS^MC^CM^ZS^ZM^XT^XD^R0^R1^R2^R3^';
  begin
    S := Switch;
    System.Delete(S, 1, 1);
    CASE Length(S) OF
      0 : Result := False;
      1 : begin
        T.Preset := Pos(S, '12345');
        IF T.Preset > 0 THEN
          begin
            IF CheckPreset(T.Preset) THEN
              begin
                ZeroSwitches(T);
                T.AnySwitch := True;
                Result := True;
              end
            ELSE Result := False;
          end
        ELSE
          begin
            Result := False;
            WriteLx(Format('"%s" is not a valid switch', [Switch]));
          end;
      end;
      2 : begin
        Result := True;
        IF T.Preset > 0 THEN Exit;
        P := Pos('^'+UpperCase(S)+'^', twos);
        CASE P OF
          01 : T.AtA := 1;
          04 : T.AtS := 1;
          07 : T.AtH := 1;
          10 : T.AtR := 1;
          13 : T.AtA := -1;
          16 : T.AtS := -1;
          19 : T.AtH := -1;
          22 : T.AtR := -1;
          25 : T.ModAct := c_fromsys;
          28 : T.CreAct := c_fromsys;
          31 : T.ModAct := c_fromother;
          34 : T.CreAct := c_fromother;
          37 : T.zersec := True;
          40 : T.zermin := True;
          43 : T.exDate := True;
          46 : T.exTime := True;
          49 : T.Rept := 0;
          52 : T.Rept := 1;
          55 : T.Rept := 2;
          58 : T.Rept := 3;
          ELSE
            begin
              Result := False;
              WriteLx(Format('"%s" is not a valid switch',[Switch]));
            end;
          END;
          IF Result AND (P < 37) THEN T.AnySwitch := True;
        end;
        ELSE
          begin
            Result := True;
            IF T.Preset > 0 THEN Exit;
            IF AnsiStartsText('S:', S) THEN
              begin
                System.Delete(S, 1, 2);
                T.Seq := StrToIntDef(S, -1);
                IF T.Seq < 0 THEN
                  begin
                    T.Seq := 0;
                    Result := False;
                    WriteLx(Format('"%s" is not a valid switch',
                      [Switch]));
                  end
                ELSE IF T.Seq > 1800 THEN
                  begin
                    T.Seq := 0;
                    Result := False;
                    WriteLx('Sequential increment is limited to '+
                      'one hour');
                  end;
              end
            ELSE IF AnsiStartsText('T:', S) THEN
              begin
                System.Delete(S, 1, 2);
                IF NOT TryStrToTime(S, T.SetTime) THEN
                  begin
                    Result := False;
                    WriteLx(Format('"%s" is not a valid time', [S]));
                  end;
              end
            ELSE IF AnsiStartsText('D:', S) THEN
              begin
                System.Delete(S, 1, 2);
                IF NOT TryStrToDate(S, T.SetDate) THEN
                  begin
                    Result := False;
                    WriteLx(Format('"%s" is not a valid date', [S]));
                  end;
              end
            ELSE
              begin
                Result := False;
                WriteLx(Format('"%s" is not a valid switch',
                  [Switch]));
              end;
          end;
      end;
  end;

  function CheckSpec(const Spec : String) : Boolean;
  VAR
    FindHan  : THandle;
    FindData : TWin32FindData;
    Drive    : String;
    Mapped   : String;
  begin
    Drive := IncludeTrailingBackslash(ExtractFileDrive(
      ExpandFilename(Spec)));
    CASE GetDriveType(PChar(Drive)) OF
      DRIVE_REMOTE : begin
        Result := False;
        Mapped := GetRemote(Drive);
        IF Mapped = Drive THEN
          WriteLx(Format('Network drives not supported (%s)',
            [Drive]))
        ELSE
          WriteLx(Format('Network drives not supported (%s, mapped '+
            'to %s)', [Mapped, Drive]));
      end;
      DRIVE_CDROM : begin
        Result := False;
        WriteLx(Format('CD-ROM drives not supported (%s)',
            [Drive]));
      end;
      ELSE begin
        FindHan := FindFirstFile(PChar(Spec), FindData);
        IF FindHan = INVALID_HANDLE_VALUE THEN
          begin
            Result := False;
            IF CanWrite THEN
              WriteLx(Format('No files match "%s"', [Spec]));
          end
        ELSE
          begin
            Result := True;
            Windows.FindClose(FindHan);
            T.Specs := T.Specs + Spec + #13#10;
          end;
      end;
    END;
  end;

begin
  Msg := '';
  T.AnySwitch := False;
  Result  := True;
  T.Preset  := 0;
  ZeroSwitches(T);
  T.Specs := '';
  FOR N := 1 TO ParamCount DO
    IF ParamStr(N)[1] IN ['-','/'] THEN
      begin
        IF NOT GoodSwitch(ParamStr(N)) THEN
          Result := False;
      end
    ELSE
      IF (NOT CheckSpec(ParamStr(N))) AND CanWrite THEN
        Result := False;
  IF T.AnySwitch THEN
    begin
      IF T.ExDate OR T.ExTime THEN T.Seq := 0;
      IF T.ExTime THEN
        begin
          T.ZerSec := False;
          T.ZerMin := False;
        end;
      IF NOT ((T.ModAct = c_fromSys) OR (T.creAct = c_fromSys)) THEN
        begin
          T.Seq := 0;
          T.SetDate := -1;
          T.SetTime := -1;
          T.ZerSec := False;
          T.ZerMin := False;
          T.ExDate := False;
          T.ExTime := False;
        end;
      IF T.ExDate THEN T.SetDate := -1;
      IF T.ExTime THEN T.SetTime := -1;
    end;
  IF (NOT CanWrite) AND (Msg <> '') THEN
    MessageBox(0, PChar(Msg), 'FileTouch command-line error',
      MB_OK OR MB_ICONSTOP);
end;

procedure CopyShared(const fromT : TShared; VAR toT : TShared);
// Copy one shared data record to another
begin
  ToT.Preset     := FromT.Preset   ;
  ToT.PresetF    := FromT.PresetF  ;
  ToT.AtA        := FromT.AtA      ;
  ToT.AtS        := FromT.AtS      ;
  ToT.AtH        := FromT.AtH      ;
  ToT.AtR        := FromT.AtR      ;
  ToT.ModAct     := FromT.ModAct   ;
  ToT.CreAct     := FromT.CreAct   ;
  ToT.SetTime    := FromT.SetTime  ;
  ToT.SetDate    := FromT.SetDate  ;
  ToT.ZerSec     := FromT.ZerSec   ;
  ToT.ZerMin     := FromT.ZerMin   ;
  ToT.ExDate     := FromT.ExDate   ;
  ToT.ExTime     := FromT.ExTime   ;
  ToT.Seq        := FromT.Seq      ;
  ToT.AnySwitch  := FromT.AnySwitch;
  ToT.Specs      := FromT.Specs    ;
end;

function Perfect(T : TShared) : String;
// Take a shared data area and render it into a perfect command-
// line. This is used when FTOUCH passes control to FileTouch.
// It does not simply fill in the shared data area and SHARE it
// because the existence or non-existence of the shared data
// area is used to handle running FileTouch as just one instance.
begin
  Result := '';
  Result := StringReplace(T.Specs, #13#10, '" "', [rfReplaceAll]);
  IF Result <> '' THEN
    Result := '"' + Result + '"';
  IF T.Preset > 0 THEN
    Result := Result + ' /' + IntToStr(T.Preset)
  ELSE
    begin
      CASE T.AtA OF
        -1 : Result := Result + ' /A-';
        1  : Result := Result + ' /A+';
      END;
      CASE T.AtS OF
        -1 : Result := Result + ' /S-';
        1  : Result := Result + ' /S+';
      END;
      CASE T.AtH OF
        -1 : Result := Result + ' /H-';
        1  : Result := Result + ' /H+';
      END;
      CASE T.AtR OF
        -1 : Result := Result + ' /R-';
        1  : Result := Result + ' /R+';
      END;
      CASE T.ModAct OF
        c_fromSys   : Result := Result + ' /MS';
        c_fromOther : Result := Result + ' /MC';
      END;
      CASE T.CreAct OF
        c_fromSys   : Result := Result + ' /CS';
        c_fromOther : Result := Result + ' /CM';
      END;
      IF T.SetDate > -1 THEN
        Result := Result + ' /D:'+DateToStr(T.SetDate);
      IF T.SetTime > -1 THEN
        Result := Result + ' /T:'+TimeToStr(T.SetTime);
      IF T.ZerSec THEN Result := Result + ' /ZS';
      IF T.ZerMin THEN Result := Result + ' /ZM';
      IF T.ExDate THEN Result := Result + ' /XD';
      IF T.ExTime THEN Result := Result + ' /XT';
      IF T.Seq > 0 THEN Result := Result + ' /S:'+IntToSTr(T.Seq);
    end;
end;

procedure ShowSyntax(CanWrite : Boolean);
// List all the command-line switches. If under FTOUCH, just write
// them to the console. If under FileTouch, show in a messagebox.
VAR Msg : String;

  procedure WriteLx(const S : String);
  begin
    IF CanWrite THEN System.WriteLn(S)
    ELSE Msg := Msg + S + #13#10;
  end;

begin
  Msg := '';
  IF CanWrite THEN
    WriteLx('Command-line switches for FTOUCH:')
  ELSE
    WriteLx('Command-line switches for FileTouch:');
  WriteLx('------------------------------------');
  IF CanWrite THEN
    WriteLx('  /R0..R3 - Reporting: 0=none, 1=errors only (default), 2=filenames, 3=details.');
  WriteLx('  /1 .. /5 - Apply the specified preset. All other switches are ignored.');
  WriteLx('');
  WriteLx('  /T:time, /D:date - Use the specified time or date for system date/time.');
  WriteLx('  /ZS, /ZM - Zero seconds or minutes before using system date/time.');
  WriteLx('  /MS, /CS - Set Modified or Created date/time from system date/time.');
  WriteLx('  /MC, /CM - Set Modified from Created or Created from Modified.');
  WriteLx('');
  WriteLx('  /XT - Don''t change the time portion of date/time.');
  WriteLx('  /XD - Don''t change the date portion of date/time.');
  WriteLx('  /S:# - Increment successive date/time stamps by the specified number');
  WriteLx('    of seconds, up to 1/2 hour (1800).');
  WriteLx('');
  WriteLx('  /A+, /S+, /H+, /R+ - Add the specified file attribute.');
  WriteLx('  /A-, /S-, /H-, /R- - Remove the specified file attribute.');
  WriteLx('');
  IF CanWrite THEN
    begin
      WriteLx('  If one or more wildcard file specifications are included with command-line');
      WriteLx('    switches, FTOUCH will simply apply the specified actions to the files.');
      WriteLx('  For file specifications without switches or vice versa, FTOUCH will load');
      WriteLx('    FileTouch and apply the switches or load the files.');
    end
  ELSE
    begin
      WriteLx('  FileTouch will open with its controls set to match the switches and its');
      WriteLx('    file list loaded with the specified files.');
      MessageBox(0, PChar(Msg), 'FileTouch command-line switches',
        MB_OK OR MB_ICONINFORMATION);
    end;
end;

procedure ZeroAll(VAR T : TShared);
// Set all fields of T to defaults and clear the filespecs
begin
  ZeroSwitches(T);
  T.Specs := '';
end;

procedure ZeroSwitches(VAR T : TShared);
// Set all fields of T to defaults
begin
  WITH T DO
    begin
      Rept    := 1;
      AtA     := 0;
      AtS     := 0;
      AtH     := 0;
      AtR     := 0;
      ModAct  := 0;
      CreAct  := 0;
      ZerSec  := False;
      ZerMin  := False;
      ExDate  := False;
      ExTime  := False;
      Seq     := 0;
      SetTime := -1;
      SetDate := -1;
      AnySwitch := False;
    end;
end;

end.
