unit tfuncsu;

// Copyright  2002 by Ziff Davis Media, Inc.
// Written by Neil J. Rubenking
{$warn UNIT_PLATFORM off}
{$warn SYMBOL_PLATFORM off}
{$warn UNSAFE_CODE off}
{$warn UNSAFE_TYPE off}
{$warn UNSAFE_CAST off}

interface

uses Windows, SysUtils, Graphics, Controls, ComCtrls, Forms;

procedure CycleButtons(TB : TToolbar);
function GetFolder(F : TForm; const S, Title : String;
  VAR GotS : String) : Boolean;
procedure LoadImageList(IL : TImageList; iFrom,
  iTo: Integer); overload;
procedure LoadImageList(IL : TImageList; iA : ARRAY
  OF Integer); overload;

implementation

uses filectrl, commctrl, shlobj, ShellApi, ActiveX;

procedure LoadImageList(IL : TImageList; iFrom, iTo: Integer);
// Load the image list with resource bitmaps in the specified
// range. By doing so, we save space and avoid complications
// with earlier versions of COMCTL32.DLL
VAR
  TB : TBitmap;
  N  : Integer;
begin
  TB := TBitmap.Create;
  try
    IL.Clear;
    FOR N := iFrom TO iTo DO
      begin
        TB.Handle := LoadImage(hInstance, MakeIntResource(N),
          IMAGE_BITMAP, IL.Width, IL.Height, lr_DefaultColor);
        IL.AddMasked(TB, clRed);
      end;
  finally
    TB.Free;
  end;
end;

procedure LoadImageList(IL : TImageList; iA : ARRAY OF Integer);
// Load the image list with resource bitmaps specified in the
// array. By doing so, we save space and avoid complications
// with earlier versions of COMCTL32.DLL
VAR
  TB : TBitmap;
  N  : Integer;
begin
  TB := TBitmap.Create;
  try
    IL.Clear;
    FOR N := Low(iA) TO High(iA) DO
      begin
        TB.Handle := LoadImage(hInstance, MakeIntResource(iA[N]),
          IMAGE_BITMAP, IL.Width, IL.Height, lr_DefaultColor);
        IL.AddMasked(TB, clRed);
      end;
  finally
    TB.Free;
  end;
end;

procedure CycleButtons(TB : TToolbar);
// FileTouch has numerous toolbars that can have either NO buttons
// down or exactly one. This routine cycles through the possible
// states, depressing each button in turn, or raising all if the
// last button has been reached. It skips disabled buttons.
VAR N, Btn : Integer;
begin
  Btn := -1;
  FOR N := 0 TO TB.ButtonCount-1 DO
    IF TB.Buttons[N].Down THEN
      begin
        Btn := N;
        Break;
      end;
  Inc(Btn);
  IF (Btn < TB.ButtonCount) AND (NOT TB.Buttons[Btn].Enabled) THEN
    Inc(Btn);
  FOR N := 0 TO TB.ButtonCount-1 DO
    WITH TB.Buttons[N] DO
      Down := N=Btn;
end;

type
  // Used by the SHBrowseForFolder callback
  PPassData = ^TPassData;
  TPassData = record
    buff : ARRAY[0..MAX_PATH] OF Char;
    Wid  : Integer;
    Can  : TCanvas;
  end;

function BrowseCallBackProc(hWindow : HWND; uMsg : Integer;
  lParam : LPARAM; lpData : PPassData) : Integer; stdcall;
VAR
  buffer : ARRAY[0..MAX_PATH] OF Char;
  Dummy  : DWord;
  SF     : SHFileInfo;
  R      : TRect;
  SDisp  : String;
begin
  Result := 0;
  IF uMsg = BFFM_INITIALIZED THEN
    begin
      GetClientRect(hWindow, R);
      lpData.Wid := R.Right - R.Left - 20;
      StrCopy(Buffer, lpData.buff);
      StrPCopy(Buffer, ExcludeTrailingBackslash(StrPas(Buffer)));
      // Set the starting directory
      IF (StrLen(Buffer) = 2) AND (Buffer[1] = ':') THEN
        StrCat(Buffer, '\');
      SendMessage(hWindow, BFFM_SETSELECTION, 1, Integer(@Buffer));
    end
  ELSE IF uMsg = BFFM_SELCHANGED THEN
    begin
      FillChar(SF, SizeOf(SF), 0);
      Dummy := 0;
      SHGetFileInfo(PChar(lParam), Dummy, SF,
        SizeOf(SF), SHGFI_PIDL	OR SHGFI_TYPENAME);
      IF AnsiCompareText(SF.szTypeName, 'Folder Shortcut') = 0 THEN
        begin
          SendMessage(hWindow, BFFM_ENABLEOK, 0, 0);
          StrCopy(Buffer, 'Folder shortcut, not a valid selection.');
          SendMessage(hWindow, BFFM_SETSTATUSTEXT, 0,
            Integer(@Buffer));
        end
      ELSE
        begin
          SHGetPathFromIDList(pItemIDList(lParam), buffer);
          IF Buffer[0] = #0 THEN
            SendMessage(hWindow, BFFM_ENABLEOK, 0, 0);
          // Display the current selection
          SDisp := MinimizeName(StrPas(Buffer), lpData.Can,
            lpData.Wid);
          StrPCopy(Buffer, SDisp);
          SendMessage(hWindow, BFFM_SETSTATUSTEXT, 0,
            Integer(@Buffer));
        end;
    end
  ELSE
    begin
      StrPCopy(Buffer, Format('"%s" is not a valid folder name',
        [PChar(lParam)]));
      SendMessage(hWindow, BFFM_SETSTATUSTEXT, 0, Integer(@Buffer));
      MessageBeep(MB_ICONSTOP);
      Result := 1;
    end;
end;

function GetFolder(F : TForm; const S, Title : String;
  VAR GotS : String) : Boolean;
VAR
  TB     : TBrowseInfo;
  IDL    : PItemIdList;
  buffer : ARRAY[0..MAX_PATH] OF Char;
  TPass  : TPassData;
begin
  FillChar(TB, SizeOf(TB), 0);
  // Connect it to our window
  TB.hwndOwner := F.Handle;
  // Set the browse-dialog's title
  TB.lpszTitle := PChar(Title);
  TB.ulFlags   := BIF_RETURNONLYFSDIRS OR BIF_RETURNFSANCESTORS
    OR BIF_STATUSTEXT ;
  // Set the browse callback procedure
  TB.lpfn      := @BrowseCallBackProc;
  // point the lParam at the desired starting folder
  // point the lParam at the desired starting folder
  StrPCopy(TPass.buff, S);
  TPass.Can := F.Canvas;
  TPass.Wid := 0;
  TB.lParam    := Integer(@TPass);
  IDL := ShBrowseForFolder(TB);
  IF IDL <> nil THEN
    begin
      SHGetPathFromIDList(IDL, Buffer);
      CoTaskMemFree(IDL);
      GotS   := IncludeTrailingBackSlash(StrPas(Buffer));
      Result := True;
    end
  ELSE Result := False;
end;

end.
